@@ -1,6 +1,7 @@
 __author__    = 'Tim Tomes (@lanmaster53)'
 
 from datetime import datetime
+from pathlib import Path
 from urllib.parse import urljoin
 import errno
 import imp
@@ -15,10 +16,10 @@ import builtins
 
 # import framework libs
 from recon.core import framework
-from recon.core.constants import BANNER
+from recon.core.constants import BANNER, BANNER_SMALL
 
 # set the __version__ variable based on the VERSION file
-exec(open(os.path.join(sys.path[0], 'VERSION')).read())
+exec(open(os.path.join(Path(os.path.abspath(__file__)).parents[2], 'VERSION')).read())
 
 # using stdout to spool causes tab complete issues
 # therefore, override print function
@@ -31,7 +32,8 @@ def spool_print(*args, **kwargs):
         if framework.Framework._spool:
             framework.Framework._spool.write(f"{args[0]}{os.linesep}")
             framework.Framework._spool.flush()
-        if 'console' in kwargs and kwargs['console'] is False:
+        # disable terminal output for server jobs
+        if framework.Framework._mode == Mode.JOB:
             return
         # new print function must still use the old print function via the backup
         builtins._print(*args, **kwargs)
@@ -48,7 +50,7 @@ class Recon(framework.Framework):
 
     repo_url = 'https://raw.githubusercontent.com/lanmaster53/recon-ng-modules/master/'
 
-    def __init__(self, check=True, analytics=True, marketplace=True):
+    def __init__(self, check=True, analytics=True, marketplace=True, accessible=False):
         framework.Framework.__init__(self, 'base')
         self._name = 'recon-ng'
         self._prompt_template = '{}[{}] > '
@@ -57,6 +59,7 @@ class Recon(framework.Framework):
         self._check = check
         self._analytics = analytics
         self._marketplace = marketplace
+        self._accessible = accessible
         # set path variables
         self.app_path = framework.Framework.app_path = sys.path[0]
         self.core_path = framework.Framework.core_path = os.path.join(self.app_path, 'core')
@@ -67,7 +70,7 @@ class Recon(framework.Framework):
 
     def start(self, mode, workspace='default'):
         # initialize framework components
-        self._mode = mode
+        self._mode = framework.Framework._mode = mode
         self._init_global_options()
         self._init_home()
         self._init_workspace(workspace)
@@ -118,9 +121,14 @@ class Recon(framework.Framework):
             self.alert('Version check disabled.')
 
     def _print_banner(self):
-        banner_len = len(max(BANNER.split(os.linesep), key=len))
-        print(BANNER)
-        print('{0:^{1}}'.format(f"{framework.Colors.O}[{self._name} v{__version__}, {__author__}]{framework.Colors.N}", banner_len + 8))
+        banner = BANNER
+        banner_len = len(max(banner.split(os.linesep), key=len))
+        author = '{0:^{1}}'.format(f"{framework.Colors.O}[{self._name} v{__version__}, {__author__}]{framework.Colors.N}", banner_len + 8)
+        if self._accessible:
+            banner = BANNER_SMALL
+            author = f"{framework.Colors.O}{self._name}, version {__version__}, by {__author__}{framework.Colors.N}"
+        print(banner)
+        print(author)
         print('')
         counts = [(len(self._loaded_category[x]), x) for x in self._loaded_category]
         if counts:
@@ -200,7 +208,7 @@ class Recon(framework.Framework):
         self._load_modules()
         return True
 
-    def delete_workspace(self, workspace):
+    def remove_workspace(self, workspace):
         path = os.path.join(self.spaces_path, workspace)
         try:
             shutil.rmtree(path)
@@ -211,12 +219,12 @@ class Recon(framework.Framework):
         return True
 
     def _get_workspaces(self):
-        dirnames = []
-        path = os.path.join(self.spaces_path)
+        workspaces = []
+        path = self.spaces_path
         for name in os.listdir(path):
             if os.path.isdir(os.path.join(path, name)):
-                dirnames.append(name)
-        return dirnames
+                workspaces.append(name)
+        return workspaces
 
     def _get_snapshots(self):
         snapshots = []
@@ -226,24 +234,25 @@ class Recon(framework.Framework):
         return snapshots
 
     def _create_db(self):
-        self.query('CREATE TABLE IF NOT EXISTS domains (domain TEXT, module TEXT)')
-        self.query('CREATE TABLE IF NOT EXISTS companies (company TEXT, description TEXT, module TEXT)')
-        self.query('CREATE TABLE IF NOT EXISTS netblocks (netblock TEXT, module TEXT)')
-        self.query('CREATE TABLE IF NOT EXISTS locations (latitude TEXT, longitude TEXT, street_address TEXT, module TEXT)')
-        self.query('CREATE TABLE IF NOT EXISTS vulnerabilities (host TEXT, reference TEXT, example TEXT, publish_date TEXT, category TEXT, status TEXT, module TEXT)')
-        self.query('CREATE TABLE IF NOT EXISTS ports (ip_address TEXT, host TEXT, port TEXT, protocol TEXT, module TEXT)')
-        self.query('CREATE TABLE IF NOT EXISTS hosts (host TEXT, ip_address TEXT, region TEXT, country TEXT, latitude TEXT, longitude TEXT, module TEXT)')
-        self.query('CREATE TABLE IF NOT EXISTS contacts (first_name TEXT, middle_name TEXT, last_name TEXT, email TEXT, title TEXT, region TEXT, country TEXT, module TEXT)')
-        self.query('CREATE TABLE IF NOT EXISTS credentials (username TEXT, password TEXT, hash TEXT, type TEXT, leak TEXT, module TEXT)')
-        self.query('CREATE TABLE IF NOT EXISTS leaks (leak_id TEXT, description TEXT, source_refs TEXT, leak_type TEXT, title TEXT, import_date TEXT, leak_date TEXT, attackers TEXT, num_entries TEXT, score TEXT, num_domains_affected TEXT, attack_method TEXT, target_industries TEXT, password_hash TEXT, password_type TEXT, targets TEXT, media_refs TEXT, module TEXT)')
-        self.query('CREATE TABLE IF NOT EXISTS pushpins (source TEXT, screen_name TEXT, profile_name TEXT, profile_url TEXT, media_url TEXT, thumb_url TEXT, message TEXT, latitude TEXT, longitude TEXT, time TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS domains (domain TEXT, notes TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS companies (company TEXT, description TEXT, notes TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS netblocks (netblock TEXT, notes TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS locations (latitude TEXT, longitude TEXT, street_address TEXT, notes TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS vulnerabilities (host TEXT, reference TEXT, example TEXT, publish_date TEXT, category TEXT, status TEXT, notes TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS ports (ip_address TEXT, host TEXT, port TEXT, protocol TEXT, banner TEXT, notes TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS hosts (host TEXT, ip_address TEXT, region TEXT, country TEXT, latitude TEXT, longitude TEXT, notes TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS contacts (first_name TEXT, middle_name TEXT, last_name TEXT, email TEXT, title TEXT, region TEXT, country TEXT, phone TEXT, notes TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS credentials (username TEXT, password TEXT, hash TEXT, type TEXT, leak TEXT, notes TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS leaks (leak_id TEXT, description TEXT, source_refs TEXT, leak_type TEXT, title TEXT, import_date TEXT, leak_date TEXT, attackers TEXT, num_entries TEXT, score TEXT, num_domains_affected TEXT, attack_method TEXT, target_industries TEXT, password_hash TEXT, password_type TEXT, targets TEXT, media_refs TEXT, notes TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS pushpins (source TEXT, screen_name TEXT, profile_name TEXT, profile_url TEXT, media_url TEXT, thumb_url TEXT, message TEXT, latitude TEXT, longitude TEXT, time TEXT, notes TEXT, module TEXT)')
         self.query('CREATE TABLE IF NOT EXISTS profiles (username TEXT, resource TEXT, url TEXT, category TEXT, notes TEXT, module TEXT)')
-        self.query('CREATE TABLE IF NOT EXISTS repositories (name TEXT, owner TEXT, description TEXT, resource TEXT, category TEXT, url TEXT, module TEXT)')
+        self.query('CREATE TABLE IF NOT EXISTS repositories (name TEXT, owner TEXT, description TEXT, resource TEXT, category TEXT, url TEXT, notes TEXT, module TEXT)')
         self.query('CREATE TABLE IF NOT EXISTS dashboard (module TEXT PRIMARY KEY, runs INT)')
-        self.query('PRAGMA user_version = 8')
+        self.query('PRAGMA user_version = 10')
 
     def _migrate_db(self):
         db_version = lambda self: self.query('PRAGMA user_version')[0][0]
+        db_orig = db_version(self)
         if db_version(self) == 0:
             # add mname column to contacts table
             tmp = self.get_random_str(20)
@@ -294,7 +303,7 @@ class Recon(framework.Framework):
             self.query('CREATE TABLE IF NOT EXISTS profiles (username TEXT, resource TEXT, url TEXT, category TEXT, notes TEXT, module TEXT)')
             self.query('PRAGMA user_version = 6')
         if db_version(self) == 6:
-            # add profile table
+            # add repositories table
             self.query('CREATE TABLE IF NOT EXISTS repositories (name TEXT, owner TEXT, description TEXT, resource TEXT, category TEXT, url TEXT, module TEXT)')
             self.query('PRAGMA user_version = 7')
         if db_version(self) == 7:
@@ -302,6 +311,20 @@ class Recon(framework.Framework):
             self.query('ALTER TABLE leaks ADD COLUMN password_type TEXT')
             self.query('UPDATE leaks SET password_type=\'unknown\'')
             self.query('PRAGMA user_version = 8')
+        if db_version(self) == 8:
+            # add banner column to ports table
+            self.query('ALTER TABLE ports ADD COLUMN banner TEXT')
+            # add notes column to all tables
+            for table in ['domains', 'companies', 'netblocks', 'locations', 'vulnerabilities', 'ports', 'hosts', 'contacts', 'credentials', 'leaks', 'pushpins', 'profiles', 'repositories']:
+                if 'notes' not in [x[0] for x in self.get_columns(table)]:
+                    self.query(f"ALTER TABLE {table} ADD COLUMN notes TEXT")
+            self.query('PRAGMA user_version = 9')
+        if db_version(self) == 9:
+            # add phone column to contacts table
+            self.query('ALTER TABLE contacts ADD COLUMN phone TEXT')
+            self.query('PRAGMA user_version = 10')
+        if db_orig != db_version(self):
+            self.alert(f"Database upgraded to version {db_version(self)}.")
 
     #==================================================
     # MODULE METHODS
@@ -609,7 +632,13 @@ class Recon(framework.Framework):
 
     def _do_workspaces_list(self, params):
         '''Lists existing workspaces'''
-        self.table([[x] for x in self._get_workspaces()], header=['Workspaces'])
+        rows = []
+        for workspace in self._get_workspaces():
+            db_path = os.path.join(self.spaces_path, workspace, 'data.db')
+            modified = datetime.fromtimestamp(os.path.getmtime(db_path)).strftime('%Y-%m-%d %H:%M:%S')
+            rows.append((workspace, modified))
+        rows.sort(key=lambda x: x[0])
+        self.table(rows, header=['Workspaces', 'Modified'])
 
     def _do_workspaces_create(self, params):
         '''Creates a new workspace'''
@@ -619,21 +648,24 @@ class Recon(framework.Framework):
         if not self._init_workspace(params):
             self.output(f"Unable to create '{params}' workspace.")
 
-    def _do_workspaces_select(self, params):
-        '''Selects an existing workspace'''
+    def _do_workspaces_load(self, params):
+        '''Loads an existing workspace'''
         if not params:
-            self._help_workspaces_select()
+            self._help_workspaces_load()
             return
-        if not self._init_workspace(params):
-            self.output(f"Unable to initialize '{params}' workspace.")
+        if params in self._get_workspaces():
+            if not self._init_workspace(params):
+                self.output(f"Unable to initialize '{params}' workspace.")
+        else:
+            self.output('Invalid workspace name.')
 
-    def _do_workspaces_delete(self, params):
-        '''Deletes an existing workspace'''
+    def _do_workspaces_remove(self, params):
+        '''Removes an existing workspace'''
         if not params:
-            self._help_workspaces_delete()
+            self._help_workspaces_remove()
             return
-        if not self.delete_workspace(params):
-            self.output(f"Unable to delete '{params}' workspace.")
+        if not self.remove_workspace(params):
+            self.output(f"Unable to remove '{params}' workspace.")
 
     def do_snapshots(self, params):
         '''Manages workspace snapshots'''
@@ -676,10 +708,10 @@ class Recon(framework.Framework):
         else:
             self.error(f"No snapshot named '{params}'.")
 
-    def _do_snapshots_delete(self, params):
-        '''Deletes an existing snapshot'''
+    def _do_snapshots_remove(self, params):
+        '''Removes an existing snapshot'''
         if not params:
-            self._help_snapshots_delete()
+            self._help_snapshots_remove()
             return
         if params in self._get_snapshots():
             os.remove(os.path.join(self.workspace, params))
@@ -716,8 +748,8 @@ class Recon(framework.Framework):
             # send analytics information
             mod_loadpath = os.path.abspath(sys.modules[y.__module__].__file__)
             self._send_analytics(mod_dispname)
-            # return the loaded module if in command line mode
-            if self._mode == Mode.CLI:
+            # return the loaded module if not in console mode
+            if self._mode != Mode.CONSOLE:
                 return y
             # begin a command loop
             y.prompt = self._prompt_template.format(self.prompt[:-3], mod_dispname.split('/')[-1])
@@ -778,13 +810,13 @@ class Recon(framework.Framework):
         print(getattr(self, '_do_workspaces_create').__doc__)
         print(f"{os.linesep}Usage: workspace create <name>{os.linesep}")
 
-    def _help_workspaces_select(self):
-        print(getattr(self, '_do_workspaces_select').__doc__)
-        print(f"{os.linesep}Usage: workspace select <name>{os.linesep}")
+    def _help_workspaces_load(self):
+        print(getattr(self, '_do_workspaces_load').__doc__)
+        print(f"{os.linesep}Usage: workspace load <name>{os.linesep}")
 
-    def _help_workspaces_delete(self):
-        print(getattr(self, '_do_workspaces_delete').__doc__)
-        print(f"{os.linesep}Usage: workspace delete <name>{os.linesep}")
+    def _help_workspaces_remove(self):
+        print(getattr(self, '_do_workspaces_remove').__doc__)
+        print(f"{os.linesep}Usage: workspace remove <name>{os.linesep}")
 
     def help_snapshots(self):
         print(getattr(self, 'do_snapshots').__doc__)
@@ -794,9 +826,9 @@ class Recon(framework.Framework):
         print(getattr(self, '_do_snapshots_load').__doc__)
         print(f"{os.linesep}Usage: snapshots load <name>{os.linesep}")
 
-    def _help_snapshots_delete(self):
-        print(getattr(self, '_do_snapshots_delete').__doc__)
-        print(f"{os.linesep}Usage: snapshots delete <name>{os.linesep}")
+    def _help_snapshots_remove(self):
+        print(getattr(self, '_do_snapshots_remove').__doc__)
+        print(f"{os.linesep}Usage: snapshots remove <name>{os.linesep}")
 
     #==================================================
     # COMPLETE METHODS
@@ -836,9 +868,9 @@ class Recon(framework.Framework):
         return []
     _complete_workspaces_create = _complete_workspaces_list
 
-    def _complete_workspaces_select(self, text, *ignored):
+    def _complete_workspaces_load(self, text, *ignored):
         return [x for x in self._get_workspaces() if x.startswith(text)]
-    _complete_workspaces_delete = _complete_workspaces_select
+    _complete_workspaces_remove = _complete_workspaces_load
 
     def complete_snapshots(self, text, line, *ignored):
         arg, params = self._parse_params(line.split(' ', 1)[1])
@@ -853,7 +885,7 @@ class Recon(framework.Framework):
 
     def _complete_snapshots_load(self, text, *ignored):
         return [x for x in self._get_snapshots() if x.startswith(text)]
-    _complete_snapshots_delete = _complete_snapshots_load
+    _complete_snapshots_remove = _complete_snapshots_load
 
     def _complete_modules_reload(self, text, *ignored):
         return []
@@ -866,7 +898,8 @@ class Mode(object):
    '''Contains constants that represent the state of the interpreter.'''
    CONSOLE = 0
    CLI     = 1
-   GUI     = 2
+   WEB     = 2
+   JOB     = 3
    
    def __init__(self):
        raise NotImplementedError('This class should never be instantiated.')
